@@ -3,6 +3,9 @@ Created on 2024-01-10
 
 @author: wf
 """
+import os
+from datetime import datetime, timezone
+
 from ngwidgets.progress import NiceguiProgressbar
 from ngwidgets.webserver import NiceGuiWebserver
 from ngwidgets.widgets import Link
@@ -98,6 +101,15 @@ class ButtonRow:
             self.achievement.level = None
         else:
             self.achievement.level = selected_level
+            # Get the current time in UTC
+            current_time_utc = datetime.now(timezone.utc)
+
+            # Convert the current time to ISO format
+            date_assessed_iso = current_time_utc.isoformat()
+
+            # Assign it to self.achievement.date_assessed_iso
+            self.achievement.date_assessed_iso = date_assessed_iso
+
         self.set_button_states(self.achievement)
         # refresh the ui
         self.row.update()
@@ -132,6 +144,35 @@ class Assessment:
         self.reset(dcm=dcm, learner=learner)
         self.setup_ui()
 
+    def store(self) -> str:
+        """
+        Store the current state of
+        the learner's achievements.
+
+        Returns(str): the path to the file
+        """
+        file_path = None
+        try:
+            # Serialize the learner object to JSON
+            learner_data_json = self.learner.to_json(indent=2)
+
+            # Determine the file path for storing the learner's data
+            filename = self.learner.file_name + ".json"
+            file_path = os.path.join(
+                self.webserver.server_config.storage_path, filename
+            )
+
+            # Write the serialized data to the file
+            with open(file_path, "w") as file:
+                file.write(learner_data_json)
+
+            if self.debug:
+                print(f"Learner data stored in {file_path}")
+
+        except Exception as ex:
+            self.webserver.handle_exception(ex, self.webserver.do_trace)
+        return file_path
+
     def reset(
         self,
         dcm: DynamicCompetenceMap,
@@ -321,6 +362,7 @@ class Assessment:
         """
         self.index_view.text = self.get_index_str()
         achievement = self.current_achievement
+        self.store()
         self.webserver.render_dcm(
             self.dcm,
             self.learner,
@@ -331,7 +373,7 @@ class Assessment:
         self.button_row.set_button_states(achievement)
         competence_element = self.competence_tree.lookup_by_path(achievement.path)
         if not competence_element:
-            ui.notify("invalid path: {achievement.path}")
+            ui.notify(f"invalid path: {achievement.path}")
             self.markdown_view.content = f"⚠️ {achievement.path}"
         else:
             if hasattr(competence_element, "path"):
