@@ -155,9 +155,11 @@ class CompetenceTree(CompetenceElement, YamlAble["CompetenceTree"]):
 
     lookup_url: Optional[str] = None
     total_levels: int = field(init=False)
+    stacked_levels: Optional[bool] = False
     aspects: List[CompetenceAspect] = field(default_factory=list)
     levels: List[CompetenceLevel] = field(default_factory=list)
     element_names: Dict[str, str] = field(default_factory=dict)
+    relative_radius: Dict[str, Tuple[float, float]] = field(default_factory=dict)
     total_elements: Dict[str, int] = field(default_factory=dict)
 
     def __post_init__(self):
@@ -174,10 +176,12 @@ class CompetenceTree(CompetenceElement, YamlAble["CompetenceTree"]):
         """
         self.path = self.id
         self.total_levels = 1
+        self.elements_by_path = {self.path: self}
         # Loop through each competence aspect and set their paths and parent references
         for aspect in self.aspects:
             aspect.competence_tree = self
             aspect.path = f"{self.id}/{aspect.id}"
+            self.elements_by_path[aspect.path] = aspect
             self.total_elements["aspects"] = self.total_elements["aspects"] + 1
             self.total_levels = 2
             for area in aspect.areas:
@@ -185,12 +189,14 @@ class CompetenceTree(CompetenceElement, YamlAble["CompetenceTree"]):
                 area.competence_tree = self
                 area.aspect = aspect
                 area.path = f"{self.id}/{aspect.id}/{area.id}"
+                self.elements_by_path[area.path] = area
                 self.total_elements["areas"] = self.total_elements["areas"] + 1
                 for facet in area.facets:
                     self.total_levels = 4
                     facet.competence_tree = self
                     facet.area = area
                     facet.path = f"{self.id}/{aspect.id}/{area.id}/{facet.id}"
+                    self.elements_by_path[facet.path] = facet
                     self.total_elements["facets"] = self.total_elements["facets"] + 1
 
     @classmethod
@@ -220,40 +226,14 @@ class CompetenceTree(CompetenceElement, YamlAble["CompetenceTree"]):
             if not lenient:
                 raise ValueError(msg)
 
-        parts = path.split("/")
-        if len(parts) < 1:
-            return None
-
-        tree_id = parts[0]
-        if tree_id != self.id:
-            handle_error(f"invalid tree_id for lookup {tree_id}")
-            return None
-        if len(parts) == 1:
-            return self
-        if len(parts) > 1:
-            aspect_id = parts[1]
-            # Retrieve the aspect
-            aspect = next(
-                (aspect for aspect in self.aspects if aspect.id == aspect_id), None
-            )
-        if aspect:
-            if len(parts) == 2:
-                return aspect
-            if len(parts) > 2:
-                area_id = parts[2]
-                area = next((area for area in aspect.areas if area.id == area_id), None)
-                if area:
-                    if len(parts) == 3:
-                        return area
-                if len(parts) > 3:
-                    facet_id = parts[3]
-                    facet = next(
-                        (facet for facet in area.facets if facet.id == facet_id), None
-                    )
-                    if facet:
-                        return facet
-        handle_error(f"invalid path for lookup {path}")
-        return None
+        element = None
+        if path not in self.elements_by_path:
+            msg = f"invalid path {path}"
+            if not lenient:
+                raise ValueError(msg)
+        else:
+            element = self.elements_by_path.get(path)
+        return element
 
     @property
     def total_valid_levels(self) -> int:
@@ -368,8 +348,8 @@ class Achievement:
     """
 
     path: str
-    level: int = None
-    score: float = None
+    level: Optional[int] = None
+    score: Optional[float] = None
     score_unit: Optional[str] = "%"
     evidence: Optional[str] = None
     date_assessed_iso: Optional[str] = None
@@ -425,10 +405,12 @@ class Learner:
     def main_id(self):
         main_id = self.learner_id
         return main_id
-    
+
     @property
     def file_name(self):
-        file_name=slugify(self.learner_id, lowercase=False, regex_pattern=r"[^\w\s\-]")
+        file_name = slugify(
+            self.learner_id, lowercase=False, regex_pattern=r"[^\w\s\-]"
+        )
         return file_name
 
     def add_achievement(self, new_achievement):
