@@ -3,11 +3,14 @@ Created on 2023-11-06
 
 @author: wf
 """
+import json
 import os
 import uuid
+from dataclasses import dataclass
 from typing import List, Optional
 from urllib.parse import urlparse
 
+import yaml
 from fastapi import HTTPException
 from fastapi.responses import HTMLResponse
 from ngwidgets.file_selector import FileSelector
@@ -40,6 +43,25 @@ class SVGRenderRequest(BaseModel):
     config: Optional[SVGConfig] = None
 
 
+@dataclass
+class ServerConfig:
+    storage_secret: str
+    storage_path: str
+
+    @classmethod
+    def from_yaml(cls, yaml_path: str):
+        if not os.path.exists(yaml_path):
+            default_storage_path = os.path.join(
+                os.path.expanduser("~"), ".dcm", "storage"
+            )
+            # Create the directory if it does not exist
+            os.makedirs(default_storage_path, exist_ok=True)
+            return cls(str(uuid.uuid4()), default_storage_path)
+        with open(yaml_path, "r") as file:
+            config_data = yaml.safe_load(file)
+        return cls(**config_data)
+
+
 class DynamicCompentenceMapWebServer(InputWebserver):
     """
     server to supply Dynamic Competence Map Visualizations
@@ -63,9 +85,16 @@ class DynamicCompentenceMapWebServer(InputWebserver):
         )
         self.examples = DynamicCompetenceMap.get_examples(markup="yaml")
         self.dcm = None
+        self.container = None
         self.learner = None
         self.assessment = None
         self.text_mode = "none"
+        config_path = os.path.join(os.environ["HOME"], ".dcm/config.yaml")
+        self.server_config = ServerConfig.from_yaml(config_path)
+
+        @app.get("/learner/{learner_slug}")
+        async def show_learner(learner_slug: str):
+            return await self.assess_learner_by_slug(learner_slug)
 
         @app.post("/svg/")
         async def render_svg(svg_render_request: SVGRenderRequest) -> HTMLResponse:
@@ -269,24 +298,20 @@ class DynamicCompentenceMapWebServer(InputWebserver):
                 text_mode=self.text_mode,
             )
             # Use the new get_java_script method to get the JavaScript
-            self.svg_view.content = svg_markup,
+            self.svg_view.content = (svg_markup,)
             self.svg_view.update()
         except Exception as ex:
             self.handle_exception(ex, self.do_trace)
 
-    async def home(self, _client: Client):
-        """Generates the home page with a selection of examples and
-        svg display
-        """
-        config=SVGConfig(with_popup=True)
+    def prepare_ui(self):
+        config = SVGConfig(with_popup=True)
         self.svg = SVG(config=config)
         java_script = self.svg.get_java_script()
 
         # Add the script using ui.add_head_html()
         ui.add_head_html(java_script)
 
-        self.setup_menu()
-
+    def show_ui(self):
         with ui.element("div").classes("w-full") as self.container:
             with ui.splitter() as splitter:
                 with splitter.before:
@@ -326,16 +351,20 @@ class DynamicCompentenceMapWebServer(InputWebserver):
                                     icon="file_open",
                                     handler=self.open_file,
                                 )
-                            self.download_button=self.tool_button(
+                            self.download_button = self.tool_button(
                                 tooltip="download",
                                 icon="download",
                                 handler=self.download,
                             )
-                            self.assess_state(False)
-            
                 with splitter.after:
                     self.svg_view = ui.html("")
-        await self.setup_footer()
+
+    async def home(self, _client: Client):
+        """Generates the home page with a selection of examples and
+        svg display
+        """
+        await self.setup_content_div(self.show_ui)
+        self.assess_state(False)
 
     def assess_learner(self, dcm, learner):
         """
@@ -346,13 +375,15 @@ class DynamicCompentenceMapWebServer(InputWebserver):
             learner(Learner): the learner to get the self assessment for
 
         """
-        if self.assessment is not None:
-            self.assessment.reset(dcm=dcm, learner=learner)
-        else:
-            with self.left_grid:
-                with ui.row() as self.assessment_row:
-                    self.assessment = Assessment(self, dcm=dcm, learner=learner)
-        self.assessment.update_achievement_view()
+        if self.container:
+            with self.container:
+                if self.assessment is not None:
+                    self.assessment.reset(dcm=dcm, learner=learner)
+                else:
+                    with self.left_grid:
+                        with ui.row() as self.assessment_row:
+                            self.assessment = Assessment(self, dcm=dcm, learner=learner)
+                self.assessment.update_achievement_view()
 
     def new_assess(self):
         """
@@ -361,6 +392,35 @@ class DynamicCompentenceMapWebServer(InputWebserver):
         self.learner = Learner(learner_id=f"{uuid.uuid4()}")
         self.assess_learner(self.dcm, self.learner)
 
+    async def assess_learner_by_slug(self, learner_slug: str):
+        """
+        Assess a learner based on the slug of the id
+
+        Args:
+            learner_slug (str): The unique slug of the learner.
+
+        Raises:
+            HTTPException: If the learner file does not exist or an error occurs.
+        """
+
+        def show():
+            learner_file = os.path.join(
+                self.server_config.storage_path, f"{learner_slug}.json"
+            )
+            if not os.path.exists(learner_file):
+                raise HTTPException(status_code=404, detail="Learner not found")
+            try:
+                with open(learner_file, "r") as file:
+                    learner_data = json.load(file)
+                    learner = Learner.from_dict(learner_data)
+            except Exception as e:
+                # Handle any exceptions related to file reading or JSON parsing
+                raise HTTPException(status_code=500, detail=str(e))
+                pass
+            self.assess(learner)
+
+        await self.setup_content_div(show())
+
     def assess(self, learner: Learner, tree_id: str = None):
         """
         run an assessment for the given learner
@@ -382,8 +442,8 @@ class DynamicCompentenceMapWebServer(InputWebserver):
         # assess_learner will render ...
         # self.render_dcm(dcm,learner=learner)
         self.assess_learner(dcm, learner)
-    
-    def assess_state(self,state:bool):
+
+    def assess_state(self, state: bool):
         if state:
             self.assessment_button.enable()
         else:
@@ -392,22 +452,19 @@ class DynamicCompentenceMapWebServer(InputWebserver):
             self.download_button.enable()
         else:
             self.download_button.disable()
-             
-    async def download(self,_args):
+
+    async def download(self, _args):
         """
         allow downloading the assessment result
         """
         try:
             with self.container:
-                if not self.learner:
+                if not self.assessment:
                     ui.notify("no active learner assessment")
                     return
-                # https://nicegui.io/documentation/download
-                json_str=self.learner.to_json(indent=2)
-                json_bytes=json_str.encode()
-                json_file=f"{self.learner.file_name}.json"
-                ui.notify(f"downloading {json_file}")
-                ui.download(json_bytes,json_file)
+                json_path = self.assessment.store()
+                ui.notify(f"downloading {json_path}")
+                ui.download(json_path)
         except Exception as ex:
             self.handle_exception(ex, self.do_trace)
 
@@ -435,3 +492,5 @@ class DynamicCompentenceMapWebServer(InputWebserver):
             DynamicCompetenceMap.examples_path(),
             self.root_path,
         ]
+        self.args.storage_secret = self.server_config.storage_secret
+        pass
