@@ -53,10 +53,10 @@ class TestDynamicCompetenceMap(Basetest):
         test looking up an element
         """
         examples = DynamicCompetenceMap.get_examples(markup="yaml")
-        example_name = "greta"
+        example_name = "greta_v2_0"
         self.assertTrue(example_name in examples)
         example = examples[example_name]
-        path = "greta/4/1/2"
+        path = "greta_v2_0/ProfessionelleSelbststeuerung/MotivationaleOrientierungen/GRETA-4-1-2"
         ct = example.competence_tree
         self.assertEqual(4, ct.total_valid_levels)
         facet = ct.lookup_by_path(path)
@@ -76,7 +76,7 @@ class TestDynamicCompetenceMap(Basetest):
         """
         _learner = Learner(learner_id="test_id")
         learner_examples = DynamicCompetenceMap.get_examples(content_class=Learner)
-        self.assertEqual(1, len(learner_examples))
+        self.assertEqual(2, len(learner_examples))
         for learner_example in learner_examples.values():
             self.assertIsInstance(learner_example, Learner)
             self.assertTrue(len(learner_example.achievements) > 0)
@@ -146,23 +146,3 @@ class TestDynamicCompetenceMap(Basetest):
                     )
                     markup_check = MarkupCheck(self, dcm)
                     markup_check.check_markup(svg_file=svg_file, svg_config=svg_config)
-
-    def test_generate_svg_from_segments(self):
-        """
-        Test the SVG generation from precalculated DonutSegment objects.
-        """
-        for markup, examples in self.example_definitions.items():
-
-            for example_name, dcm in examples.items():
-                # Now you can perform assertions to verify that the data was loaded correctly
-                self.assertIsNotNone(dcm)
-                dcm_chart = DcmChart(dcm)
-
-                # Generate SVG markup
-                svg_config = SVGConfig(with_popup=True)
-                svg_markup = dcm_chart.generate_svg_from_segments(
-                    dcm.competence_tree, config=svg_config
-                )
-                svg_file = f"/tmp/{example_name}_{markup}_segments.svg"
-                with open(svg_file, "w") as file:
-                    file.write(svg_markup)
